@@ -93,6 +93,10 @@ PyInit__psutil_sunos(void) {
 
     if (psutil_setup() != 0)
         return NULL;
+    if (psutil_posix_add_constants(mod) != 0)
+        return NULL;
+    if (psutil_posix_add_methods(mod) != 0)
+        return NULL;
 
     if (PyModule_AddIntConstant(mod, "version", PSUTIL_VERSION))
         return NULL;
